;;; ralee-helpers

; Copyright (c) 2004 Sam Griffiths-Jones
;
; This is part of RALEE -- see
; http://www.sanger.ac.uk/Users/sgj/ralee/ and the 00README file
; that should accompany this file.
;
; RALEE is free software; you can redistribute it and/or modify it
; under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; RALEE is distributed in the hope that it will be useful, but
; WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
; General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with RALEE; if not, write to the Free Software Foundation,
; Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


(defun show-structure-ps ()
  "take the current sequence, RNAfold it, and show the structure ps file"
  (interactive)
  (if (ralee-is-alignment-line)
      (let ((seqstr (ralee-get-seq-string)))
	(ralee-call-sequence-folder seqstr)
	(start-process "gv" "*messages*" "gv" "rna.ps")
	)
    )
  )


(defun fold-all-sequences ()
  "run fold-sequence on each sequence in turn"
  (interactive)
  (save-excursion
    (goto-char (point-min))
    (while (< (point) (point-max))
      (if (ralee-is-alignment-line)
	  (progn
	    (fold-sequence)
	    (forward-line)
	    )
	)
      (forward-line)
      )
    )
  )



(defun ralee-call-sequence-folder (seqstr)
  "take a sequence string, output a bracket string
dealing with gap delete and then insert"
  (let ((curbuf (current-buffer))
	structure
	split
	(gaps ())
	ungap)

    ;; remove and store the positions of gaps
    (setq split (ralee-split-string seqstr ""))
    (let ((i 0))
      (while (< i (length split))
	(let ((base (string-to-char (nth i split))))
	  ;; check base isn't nil for xemacs compatability
	  (if (and base (char-equal base ?.)) 
	      ;; If list gaps is first in the cons here, then
	      ;; you don't get a simple list.  Best to do it
	      ;; like this and then reverse the list later
	      (setq gaps (cons i gaps))
	    (setq ungap (concat ungap (nth i split)))
	    )
	  )
	(setq i (1+ i))
	)
      )

    (setq gaps (reverse gaps))

    ;; do the folding magic
    (get-buffer-create "*structures*") ;; make it if it doesn't exist
    (set-buffer "*structures*")
    (kill-region (point-min) (point-max))
    (insert ungap)
    (beginning-of-line)
    (setq beg (point))
    (end-of-line)
    (insert "\n")
    (call-process-region beg (1- (point)) "RNAfold" nil t)

    (forward-line -1)
    (beginning-of-line)

    ;; put the gaps back
    (while gaps
      (setq gap (car gaps))
      (setq gaps (cdr gaps))
      (move-to-column gap)
      (insert ".")
      )

    (beginning-of-line)
    (if (looking-at "\\([.()]+\\)")
	(setq structure (match-string 0))
      )

    (set-buffer curbuf)
;;    (print structure)
    structure
    )
  )


(defun fold-sequence (&optional startcolumn endcolumn)
  "take the current sequence, RNAfold it, and mock up an SS_cons line"
  (interactive)
  (save-excursion
    (if (ralee-is-alignment-line)
	(progn
	  (ralee-find-first-column)
	  (let ((curbuf (current-buffer))
		(seqid (ralee-get-seq-id))
		(first-col (current-column))
		(beg (point))
		seqstr
		structure
		last-col
		)
	    
	    (end-of-line)
	    ;; if startcolumn, endcolumn are not defined then
	    ;; we get the whole line
	    (setq seqstr (ralee-get-seq-string startcolumn endcolumn))
	    (setq structure (ralee-call-sequence-folder seqstr))
	    
	    (insert "\n")
	    ;; pad the ends with gaps if we're folding a subsequence
	    (if startcolumn
		(let ((i first-col))
		  (while (< i startcolumn)
		    (insert ".")
		    (setq i (1+ i))
		    )
		  )
	      )
	    (insert structure)
	    (if endcolumn
		(progn
		  (forward-line -1)
		  (end-of-line)
		  (setq last-col (current-column))
		  (forward-line 1)
		  (end-of-line)
	      
		  (while (< (current-column) (- last-col first-col))
		    (insert ".")
		    )
		  )
	      )

	    ;; insert the #=GR tag
	    (beginning-of-line)
	    (let ((string (concat "#=GR " seqid " SS  "))
		  (i 0))
	      (if (> (length string) first-col)
		  
		  (save-excursion   ;; we need to pad out all seqs to make 
		                    ;; room for the structure tag
		    (goto-char (point-min))
		    (while (< (point) (point-max))
		      (if (or (ralee-is-alignment-line) (ralee-is-markup-line))
			  (progn
			    (end-of-line)
			    (search-backward " ")
			    (let ((j 0))
			      (while (< j (- (length string) first-col))
				(insert " ")
				(setq j (1+ j))
				)
			      )
			    )
			)
		      (forward-line)
		      )
		    )
		)
	      
	      (insert string)
	      (while (< i (- first-col (length string)))
		(insert " ")
		(setq i (1+ i))
		)
	      )
	    )
	  )
      ;; else
      (message "Current line doesn't contain a sequence to fold")
      )
    )
  )
  


(defun fold-sequence-block ()
  "fold the subsequences in the current block"
  (interactive)
  (save-excursion
    (let ((start (car ralee-block))
          (end (car (cdr ralee-block))))
      (if (and start end)
          (progn
            (goto-char end)
            (let ((endcol (current-column))
                  (endrow (current-line))
                  startcol)
	      (goto-char start)
              (setq startcol (current-column))

              (while (<= (current-line) endrow)
		(fold-sequence startcol endcol)
		(setq endrow (1+ endrow))
		(forward-line 2)
		)
	      )
	    )
	)
      )
    )
  )



(defun fetch-sequence ()
  "use pfetch to get a sequence"
  (interactive)
  (let (seqid)
    (setq seqid (ralee-get-name-start-end))
    (if seqid
	(progn
	  (call-process "pfetch" nil (nth 0 seqid) t "-F" (nth 0 seqid))
	  (pop-to-buffer (nth 0 seqid))
	  (setq buffer-read-only 1)
	  (protect-mode nil)
	  (goto-char (point-min))
	  )
      )
    )
  )


(provide 'ralee-helpers)
